@@ -214,7 +214,7 @@ class GoRule13(GoRule):
 
     def __init__(self):
         super().__init__("GORULE:0000013", "Taxon-appropriate annotation check", FailMode.HARD)
-        self.non_experimental_evidence = ["ECO:0000318", "ECO:0000320", "ECO:0000321", "ECO:0000305", "ECO:0000247", "ECO:0000255", "ECO:0000266", "ECO:0000250", "ECO:0000303", "ECO:0000245", "ECO:0000304", "ECO:0000307"]
+        self.non_experimental_evidence = set(["ECO:0000318", "ECO:0000320", "ECO:0000321", "ECO:0000305", "ECO:0000247", "ECO:0000255", "ECO:0000266", "ECO:0000250", "ECO:0000303", "ECO:0000245", "ECO:0000304", "ECO:0000307", "ECO:0000501"])
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         if config.annotation_inferences is None:
@@ -253,8 +253,9 @@ class GoRule15(GoRule):
         # Cache the allowed terms
         if self.allowed_dual_species_terms is None and config.ontology is not None:
             interaction_terms = config.ontology.descendants("GO:0044419", relations=["subClassOf"], reflexive=True)
-            other_organism_terms = config.ontology.descendants("GO:0044215", relations=["subClassOf"], reflexive=True)
-            self.allowed_dual_species_terms = set(interaction_terms + other_organism_terms)
+            interspecies_interactions_regulation = config.ontology.descendants("GO:0043903", relations=["subClassOf"], reflexive=True)
+            host_cellular_component = config.ontology.descendants("GO:0018995", relations=["subClassOf"], reflexive=True)
+            self.allowed_dual_species_terms = set(interaction_terms + interspecies_interactions_regulation + host_cellular_component)
         elif config.ontology is None:
             return self._result(True)
 
@@ -371,8 +372,9 @@ class GoRule28(RepairRule):
 class GoRule29(GoRule):
 
     def __init__(self):
-        super().__init__("GORULE:0000029", "All IEAs over a year old are removed", FailMode.HARD)
+        super().__init__("GORULE:0000029", "All IEAs over a year are warned, all IEAs over two are removed", FailMode.HARD)
         self.one_year = datetime.timedelta(days=365)
+        self.two_years = datetime.timedelta(days=730)
 
     def test(self, annotation: association.GoAssociation, config: assocparser.AssocParserConfig, group=None) -> TestResult:
         evidence = annotation.evidence.type
@@ -380,19 +382,27 @@ class GoRule29(GoRule):
 
         now = datetime.datetime.today()
 
-        time_compare_delta = self.one_year
-        if group is not None and group == "ecocyc":
-            time_compare_delta = datetime.timedelta(days=2*365)
+        time_compare_delta_short = self.one_year
+        time_compare_delta_long = self.two_years
+        time_diff = now - datetime.datetime(int(date[0:4]),
+                                            int(date[4:6]),
+                                            int(date[6:8]),
+                                            0, 0, 0, 0)
 
         iea = "ECO:0000501"
-        fails = (evidence == iea and now - datetime.datetime(int(date[0:4]), int(date[4:6]), int(date[6:8]), 0, 0, 0, 0) > time_compare_delta)
-        return self._result(not fails)
+        if evidence == iea:
+            if time_diff > time_compare_delta_long:
+                return self._result(False)
+            elif time_diff > time_compare_delta_short:
+                return TestResult(ResultType.WARNING, self.title, annotation)
 
+        ## Default results we we get here.
+        return self._result(True)
 
 class GoRule30(GoRule):
 
     def __init__(self):
-        super().__init__("GORULE:0000030", "Deprecated GO_REFs are not allowed", FailMode.HARD)
+        super().__init__("GORULE:0000030", "Deprecated GO_REFs are not allowed", FailMode.SOFT)
 
     def _ref_curi_to_id(self, goref) -> str:
         """
