@@ -11,6 +11,8 @@ import networkx as nx
 import logging
 import re
 
+logger = logging.getLogger(__name__)
+
 class Ontology():
     """An object that represents a basic graph-oriented view over an ontology.
 
@@ -47,7 +49,7 @@ class Ontology():
         self.graph = graph
         if self.graph is None:
             self.graph = nx.MultiDiGraph()
-        logging.info('Graph initialized, nodes={}'.format(self.graph.nodes()))
+        logger.debug('Graph initialized, nodes={}'.format(self.graph.nodes()))
         self.xref_graph = xref_graph
 
         # obograph
@@ -62,6 +64,7 @@ class Ontology():
             self.xref_graph = payload.get('xref_graph')
             self.graphdoc = payload.get('graphdoc')
             self.all_logical_definitions = payload.get('logical_definitions')
+            self.all_property_chain_axioms = payload.get('property_chain_axioms')
 
     def __str__(self):
         return '{} handle: {} meta: {}'.format(self.id, self.handle, self.meta)
@@ -105,29 +108,29 @@ class Ontology():
         # trigger synonym cache
         self.all_synonyms()
         self.all_obsoletes()
-        
+
         # default method - wrap get_graph
         srcg = self.get_graph()
         if prefix is not None:
             srcg = srcg.subgraph([n for n in srcg.nodes() if n.startswith(prefix+":")])
         if relations is None:
-            logging.info("No filtering on "+str(self))
+            logger.info("No filtering on "+str(self))
             return srcg
-        logging.info("Filtering {} for {}".format(self, relations))
+        logger.info("Filtering {} for {}".format(self, relations))
         g = nx.MultiDiGraph()
 
         # TODO: copy full metadata
-        logging.info("copying nodes")
-        for n,d in srcg.nodes_iter(data=True):
-            g.add_node(n, attr_dict=d)
+        logger.info("copying nodes")
+        for (n,d) in srcg.nodes(data=True):
+            g.add_node(n, **d)
 
-        logging.info("copying edges")
+        logger.info("copying edges")
         num_edges = 0
-        for x,y,d in srcg.edges_iter(data=True):
+        for (x,y,d) in srcg.edges(data=True):
             if d['pred'] in relations:
                 num_edges += 1
-                g.add_edge(x,y,attr_dict=d)
-        logging.info("Filtered edges: {}".format(num_edges))
+                g.add_edge(x,y,**d)
+        logger.info("Filtered edges: {}".format(num_edges))
         return g
 
     def merge(self, ontologies):
@@ -136,18 +139,18 @@ class Ontology():
         """
         if self.xref_graph is None:
             self.xref_graph = nx.MultiGraph()
-        logging.info("Merging source: {} xrefs: {}".format(self, len(self.xref_graph.edges())))        
+        logger.info("Merging source: {} xrefs: {}".format(self, len(self.xref_graph.edges())))
         for ont in ontologies:
-            logging.info("Merging {} into {}".format(ont, self))
+            logger.info("Merging {} into {}".format(ont, self))
             g = self.get_graph()
             srcg = ont.get_graph()
             for n in srcg.nodes():
-                g.add_node(n, attr_dict=srcg.node[n])
+                g.add_node(n, **srcg.node[n])
             for (o,s,m) in srcg.edges(data=True):
-                g.add_edge(o,s,attr_dict=m)
+                g.add_edge(o,s,**m)
             if ont.xref_graph is not None:
                 for (o,s,m) in ont.xref_graph.edges(data=True):
-                    self.xref_graph.add_edge(o,s,attr_dict=m)
+                    self.xref_graph.add_edge(o,s,**m)
 
     def subgraph(self, nodes=None):
         """
@@ -218,12 +221,12 @@ class Ontology():
         """
         if subset is not None:
             subset_nodes = self.extract_subset(subset)
-            logging.info("Extracting subset: {} -> {}".format(subset, subset_nodes))
+            logger.info("Extracting subset: {} -> {}".format(subset, subset_nodes))
 
         if subset_nodes is None or len(subset_nodes) == 0:
             raise ValueError("subset nodes is blank")
         subset_nodes = set(subset_nodes)
-        logging.debug("SUBSET: {}".format(subset_nodes))
+        logger.debug("SUBSET: {}".format(subset_nodes))
 
         # Use a sub-ontology for mapping
         subont = self
@@ -241,7 +244,7 @@ class Ontology():
             ancs_in_subset = subset_nodes.intersection(ancs)
             m[n] = list(subont.filter_redundant(ancs_in_subset))
         return m
-        
+
     def filter_redundant(self, ids):
         """
         Return all non-redundant ids from a list
@@ -282,11 +285,7 @@ class Ontology():
 
     def _meta(self, nid):
         n = self.node(nid)
-        if 'meta' in n:
-            return n['meta']
-        else:
-            return {}
-
+        return n.get("meta", {})
 
     def prefixes(self):
         """
@@ -306,7 +305,7 @@ class Ontology():
         sep=':'
         if nid.startswith('http'):
             if '#' in nid:
-               sep='#' 
+               sep='#'
             else:
                 sep='/'
         parts = nid.split(sep)
@@ -320,7 +319,7 @@ class Ontology():
         """
         pfx,_ = self.prefix_fragment(nid)
         return pfx
-        
+
     def nodes(self):
         """
         Return all nodes in ontology
@@ -331,11 +330,12 @@ class Ontology():
 
     def node(self, id):
         """
-        Return a node with a given ID
+        Return a node with a given ID. If the node with the ID exists the
+        Node object is returned, otherwise None is returned.
 
         Wraps networkx by default
         """
-        return self.get_graph().node[id]
+        return self.get_graph().node.get(id, None)
 
     def has_node(self, id):
         """
@@ -354,15 +354,15 @@ class Ontology():
         """
         If stated, either CLASS, PROPERTY or INDIVIDUAL
         """
-        return self.node(id)['type']
-    
+        return self.node(id).get('type', None)
+
     def relations_used(self):
         """
         Return list of all relations used to connect edges
         """
         g = self.get_graph()
         types = set()
-        for x,y,d in g.edges_iter(data=True):
+        for (x,y,d) in g.edges(data=True):
             types.add(d['pred'])
         return list(types)
 
@@ -391,12 +391,21 @@ class Ontology():
         preds = set()
         for _,ea in graph[obj][subj].items():
             preds.add(ea['pred'])
-        logging.debug('{}->{} = {}'.format(subj,obj,preds))
+        logger.debug('{}->{} = {}'.format(subj,obj,preds))
         return preds
 
     def parents(self, node, relations=None):
         """
-        Return all direct parents of specified node.
+        Return all direct 'parents' of specified node.
+
+        Note that in the context of ontobio, 'parent' means any node that
+        is traversed in a single hop along an edge from a subject to object.
+        For example, if the ontology has an edge "finger part-of some hand", then
+        "hand" is the parent of finger.
+        This can sometimes be counter-intutitive, for example, if the ontology
+        contains has-part axioms. If the ontology has an edge
+        "X receptor activity has-part some X binding", then "X binding" is the 'parent'
+        of "X receptor activity" over a has-part edge.
 
         Wraps networkx by default.
 
@@ -410,7 +419,7 @@ class Ontology():
         """
         g = self.get_graph()
         if node in g:
-            parents = g.predecessors(node)
+            parents = list(g.predecessors(node))
             if relations is None:
                 return parents
             else:
@@ -439,7 +448,7 @@ class Ontology():
         """
         g = self.get_graph()
         if node in g:
-            children = g.successors(node)
+            children = list(g.successors(node))
             if relations is None:
                 return children
             else:
@@ -470,20 +479,16 @@ class Ontology():
             ancestor node IDs
 
         """
-        if reflexive:
-            ancs = self.ancestors(node, relations, reflexive=False)
-            ancs.append(node)
-            return ancs
-
-        g = None
-        if relations is None:
-            g = self.get_graph()
-        else:
-            g = self.get_filtered_graph(relations)
-        if node in g:
-            return list(nx.ancestors(g, node))
-        else:
-            return []
+        seen = set()
+        nextnodes = [node]
+        while len(nextnodes) > 0:
+            nn = nextnodes.pop()
+            if not nn in seen:
+                seen.add(nn)
+                nextnodes += self.parents(nn, relations=relations)
+        if not reflexive:
+            seen -= {node}
+        return list(seen)
 
     def descendants(self, node, relations=None, reflexive=False):
         """
@@ -508,19 +513,16 @@ class Ontology():
         list[str]
             descendant node IDs
         """
-        if reflexive:
-            decs = self.descendants(node, relations, reflexive=False)
-            decs.append(node)
-            return decs
-        g = None
-        if relations is None:
-            g = self.get_graph()
-        else:
-            g = self.get_filtered_graph(relations)
-        if node in g:
-            return list(nx.descendants(g, node))
-        else:
-            return []
+        seen = set()
+        nextnodes = [node]
+        while len(nextnodes) > 0:
+            nn = nextnodes.pop()
+            if not nn in seen:
+                seen.add(nn)
+                nextnodes += self.children(nn, relations=relations)
+        if not reflexive:
+            seen -= {node}
+        return list(seen)
 
 
     def equiv_graph(self):
@@ -531,12 +533,12 @@ class Ontology():
             bidirectional networkx graph of all equivalency relations
         """
         eg = nx.Graph()
-        for u,v,d in self.get_graph().edges_iter(data=True):
+        for (u,v,d) in self.get_graph().edges(data=True):
             if d['pred'] == 'equivalentTo':
                 eg.add_edge(u,v)
         return eg
-        
-        
+
+
     def traverse_nodes(self, qids, up=True, down=False, **args):
         """
         Traverse (optionally) up and (optionally) down from an input set of nodes
@@ -581,7 +583,7 @@ class Ontology():
         """
         g = self.get_filtered_graph(relations=relations, prefix=prefix)
         # note: we also eliminate any singletons, which includes obsolete classes
-        roots = [n for n in g.nodes() if len(g.predecessors(n)) == 0 and len(g.successors(n)) > 0]
+        roots = [n for n in g.nodes() if len(list(g.predecessors(n))) == 0 and len(list(g.successors(n))) > 0]
         return roots
 
     def get_level(self, level, relations=None, **args):
@@ -596,10 +598,10 @@ class Ontology():
         g = self.get_filtered_graph(relations)
         nodes = self.get_roots(relations=relations, **args)
         for i in range(level):
-            logging.info(" ITERATING TO LEVEL: {} NODES: {}".format(i, nodes))
+            logger.info(" ITERATING TO LEVEL: {} NODES: {}".format(i, nodes))
             nodes = [c for n in nodes
                      for c in g.successors(n)]
-        logging.info(" FINAL: {}".format(nodes))
+        logger.info(" FINAL: {}".format(nodes))
         return nodes
 
     def parent_index(self, relations=None):
@@ -622,7 +624,7 @@ class Ontology():
             g = self.get_filtered_graph(relations)
         l = []
         for n in g:
-            l.append([n] ++ g.predecessors(b))
+            l.append([n] + list(g.predecessors(n)))
         return l
 
     def text_definition(self, nid):
@@ -667,6 +669,25 @@ class Ontology():
         else:
             return []
 
+    def get_property_chain_axioms(self, nid):
+        """
+        Retrieves property chain axioms for a class id
+
+        Arguments
+        ---------
+        nid : str
+             Node identifier for relation to be queried
+
+        Returns
+        -------
+        PropertyChainAxiom
+        """
+        pcas = self.all_property_chain_axioms
+        if pcas is not None:
+            return [x for x in pcas if x.predicate_id == nid]
+        else:
+            return []
+
     def get_node_type(self, nid):
         n = self.node(nid)
         if 'type' in n:
@@ -710,7 +731,7 @@ class Ontology():
         dep = self._get_meta_prop(nid, 'deprecated')
         return  dep is not None and dep
 
-    
+
     def replaced_by(self, nid, strict=True):
         """
         Returns value of 'replaced by' (IAO_0100001) property for obsolete nodes
@@ -732,7 +753,7 @@ class Ontology():
             if strict:
                 raise ValueError(msg)
             else:
-                logging.error(msg)
+                logger.error(msg)
 
         return vs
 
@@ -782,7 +803,7 @@ class Ontology():
         if nid not in self.get_graph():
             self.add_node(nid)
         self._add_meta_element(nid, 'deprecated', True)
-        
+
     def _add_meta_element(self, id, k, edict):
         n = self.node(id)
         if n is None:
@@ -799,7 +820,7 @@ class Ontology():
         for n in self.nodes():
             if n in xg:
                 self._add_meta_element(n, 'xrefs', [{'val':x} for x in xg.neighbors(n)])
-        
+
     def add_parent(self, id, pid, relation='subClassOf'):
         """
         Add a new edge to the ontology
@@ -815,7 +836,7 @@ class Ontology():
         if self.xref_graph is None:
             self.xref_graph = nx.MultiGraph()
         self.xref_graph.add_edge(xref, id)
-        
+
     def add_synonym(self, syn):
         """
         Adds a synonym for a node
@@ -839,7 +860,7 @@ class Ontology():
         if 'subsets' not in meta:
             meta['subsets'] = []
         meta['subsets'].append(s)
-        
+
     def all_synonyms(self, include_label=False):
         """
         Retrieves all synonyms
@@ -864,7 +885,7 @@ class Ontology():
         Returns all obsolete nodes
         """
         return [n for n in self.nodes() if self.is_obsolete(n)]
-        
+
     def label(self, nid, id_if_null=False):
         """
         Fetches label for a node
@@ -896,7 +917,7 @@ class Ontology():
             else:
                 return None
 
-    def xrefs(self, nid, bidirectional=False):
+    def xrefs(self, nid, bidirectional=False, prefix=None):
         """
         Fetches xrefs for a node
 
@@ -911,16 +932,19 @@ class Ontology():
         ------
         list[str]
         """
+        xrefs = []
         if self.xref_graph is not None:
             xg = self.xref_graph
             if nid not in xg:
-                return []
-            if bidirectional:
-                return xg.neighbors(nid)
+                xrefs = []
             else:
-                return [x for x in xg.neighbors(nid) if xg[nid][x][0]['source'] == nid]
-
-        return []
+                if bidirectional:
+                    xrefs = list(xg.neighbors(nid))
+                else:
+                    xrefs = [x for x in xg.neighbors(nid) if xg[nid][x][0]['source'] == nid]
+                if prefix is not None:
+                    xrefs = [x for x in xrefs if self.prefix(x) == prefix]
+        return xrefs
 
 
     def resolve_names(self, names, synonyms=False, **args):
@@ -941,13 +965,13 @@ class Ontology():
         g = self.get_graph()
         r_ids = []
         for n in names:
-            logging.debug("Searching for {} syns={}".format(n,synonyms))
+            logger.debug("Searching for {} syns={}".format(n,synonyms))
             if len(n.split(":")) == 2:
                 r_ids.append(n)
             else:
                 matches = set([nid for nid in g.nodes() if self._is_match(self.label(nid), n, **args)])
                 if synonyms:
-                    logging.debug("Searching syns for {}".format(names))
+                    logger.debug("Searching syns for {}".format(names))
                     for nid in g.nodes():
                         for s in self.synonyms(nid):
                             if self._is_match(s.val, n, **args):
@@ -1072,7 +1096,7 @@ class TextDefinition(AbstractPropertyValue):
             'val': self.val,
             'xrefs': self.xrefs
         }
-        
+
 class Synonym(AbstractPropertyValue):
     """
     Represents a synonym using the OBO model
@@ -1123,12 +1147,12 @@ class Synonym(AbstractPropertyValue):
 
     def is_label(self):
         return self.pred == 'label'
-    
+
     def is_abbreviation(self, v=None):
         if v is not None:
             self.lextype = 'abbreviation'
         return self.lextype is not None and self.lextype.lower() == 'abbreviation'
-    
+
     def exact_or_label(self):
         return self.pred == 'hasExactSynonym' or self.pred == 'label'
 
@@ -1142,7 +1166,7 @@ class Synonym(AbstractPropertyValue):
             'val': self.val,
             'xrefs': self.xrefs
         }
-    
+
     def __cmp__(self, other):
         (x,y) = (str(self),str(other))
         if x > y:
@@ -1150,3 +1174,35 @@ class Synonym(AbstractPropertyValue):
         if x < y:
             return -1
         return 0
+
+class PropertyChainAxiom(object):
+    """
+    Represents a property chain axiom used to infer the existence of a property from a chain of properties.
+
+    See the OWL primer for a description of property chains: https://www.w3.org/TR/owl2-primer/#Property_Chains
+    """
+
+    def __init__(self, predicate_id, chain_predicate_ids):
+        """
+        Arguments
+        ---------
+         - predicate_id : string
+             the object property
+         - chain_predicate_ids: list
+             ordered list of chained properties
+
+        """
+        self.predicate_id = predicate_id
+        self.chain_predicate_ids = chain_predicate_ids
+
+    def __str__(self):
+        return '{} ({})'.format(self.predicate_id, " o ".join(self.chain_predicate_ids))
+
+    def as_dict(self):
+        """
+        Returns PropertyChainAxiom as obograph dict
+        """
+        return {
+            "predicateId": self.predicate_id,
+            "chainPredicateIds": self.chain_predicate_ids
+        }
